Option Explicit
' $URL: https://opensvn.csie.org/TinyCollection/PSPad/trunk/code/cvscmd.vbs $
' $Author: DonQuichote $
' $Date: 2006-08-31 21:50:27 +0200 (Thu, 31 Aug 2006) $
' $Revision: 13 $

Const module_name = "CVS" ' This name must be unique.
Const module_desc = "Routines for the CVS source code control system"
Const module_ver = "0.1" ' Version

'-----------------------------
' SETTINGS. ADAPT TO YOUR NEED.

'If True, the "Edit this script" option is added to the menu. Handy when debugging.
Const ADD_EDIT_ME_TO_MENU = True

'CVS settings
const sCVS_EXECUTABLE = ""
'Example: const sCVS_EXECUTABLE = "C:\Program Files\TortoiseCVS\cvs.exe"
'CVS commands. Variables %Input%, %File%, %FullFileName%, %FilePath%, %FileName% and %FileName2% are expanded.
'%Input% is replaced by the value of an inputbox. For the other values, see the PSPad documentation about variables.
const cCVS_ADD = "add -m ""%Input%"" ""%FileName%"""
const cCVS_UPDATE = "update ""%FileName%"""
const cCVS_COMMIT = "commit -m ""%Input%"" ""%FileName%"""
const cCVS_GET_CLEAN_COPY = "update -C ""%FileName%"""
const cCVS_LOG = "log -S ""%FileName%"""
const cCONFLICT_EDITOR = ""
'Example: const cCONFLICT_EDITOR = """C:\Program Files\CvsConflictEditor\CvsConflictEditor.Exe"" ""%File%"""

'Language settings. Accesskeys CAN be given by inserting an ampersand, but are auto-generated otherwise.
Const cMENU_EDIT_ME = "Edit this script"
const cMENU_ADD = "Add file"
const cMENU_UPDATE = "Update file"
const cMENU_COMMIT = "Co&mmit file"
const cMENU_GET_CLEAN_COPY = "Get &clean copy of file"
const cMENU_LOG = "File log"
const cMENU_CONFLICT_EDIT = "Resolve conflict"
const cMENU_PROJECT_UPDATE = "Update all in project"
const cMENU_PROJECT_GET_CLEAN_COPY = "Get all clean copies in project"
'    (END OF SETTINGS.)
'-----------------------------

Sub AddToMenu(ByVal strLabel, ByVal strFunction)
 If Len(strLabel)>0 Then
   addMenuItem strLabel, "&" & module_name, strFunction
 End If
End Sub

Function CurrentFile()
 With newEditor
     .assignActiveEditor
     CurrentFile = .fileName
 End With
End Function

Function DirectoryOf(ByVal strFullPath)
 Dim numPos
 numPos = InStrRev(strFullPath, "\")
 DirectoryOf = Left(strFullPath, numPos)
End Function

Sub DoCvsAdd()
 RunCvsCommand cCVS_ADD, DirectoryOf(CurrentFile())
End Sub

Sub DoCvsUpdate()
 RunCvsCommand cCVS_UPDATE, DirectoryOf(CurrentFile())
 ReloadCurrentFile
End Sub

Sub DoCvsUpdateAll()
 RunCvsForAll cCVS_UPDATE
End Sub

Sub DoCvsCommit()
 RunCvsCommand cCVS_COMMIT, DirectoryOf(CurrentFile())
 ReloadCurrentFile ' For keyword expansion.
End Sub

Sub DoCvsCommitAll()
 RunCvsForAll cCVS_COMMIT
End Sub

Sub DoCvsGetCleanCopy()
 RunCvsCommand cCVS_GET_CLEAN_COPY, DirectoryOf(CurrentFile())
 ReloadCurrentFile
End Sub

Sub DoCvsCleanCopyForAll()
 RunCvsForAll cCVS_GET_CLEAN_COPY
End Sub

Sub DoCvsLog()
 RunCvsCommand cCVS_LOG, DirectoryOf(CurrentFile())
End Sub

Sub EditMe()
 newEditor.openFile moduleFileName(module_name)
End Sub

' name "Init" is required, its called automatically during initialization to create menu items
' addMenuItem "Name of menu item", "Name of submenu if required", "Main"
Sub Init
 With New clsAccesskeyGenerator
     .WithAccessKey(module_name) ' So the first character of the module is not used for commands
     AddToMenu .WithAccessKey(cMENU_UPDATE), "DoCvsUpdate"
     AddToMenu .WithAccessKey(cMENU_COMMIT), "DoCvsCommit"
     AddToMenu .WithAccessKey(cMENU_GET_CLEAN_COPY), "DoCvsGetCleanCopy"
     AddToMenu .WithAccessKey(cMENU_ADD), "DoCvsAdd"
     AddToMenu .WithAccessKey(cMENU_LOG), "DoCvsLog"
     AddToMenu .WithAccessKey(cMENU_CONFLICT_EDIT), "ResolveConflict"
     If IsProjectAvailable Then
       AddToMenu "-", ""
     AddToMenu .WithAccessKey(cMENU_PROJECT_UPDATE), "DoCvsUpdateAll"
     AddToMenu .WithAccessKey(cMENU_PROJECT_GET_CLEAN_COPY), "DoCvsCleanCopyForAll"
     End If
     If ADD_EDIT_ME_TO_MENU Then
       AddToMenu "-", ""
       AddToMenu .WithAccessKey(cMENU_EDIT_ME), "EditMe"
     End If
 End With
End Sub

Function IsProjectAvailable()
 Const FIRST_VERSION_WITH_PROJECTREADING = 2181
 Dim varVersion, numPos
 varVersion = pspadVersion
 If Right(varVersion, 1)=")" Then
   varVersion = Left(varVersion, Len(varVersion) - 1)
 End If
 numPos = InStr(1, varVersion, "(")
 If numPos>0 Then
   varVersion = Mid(varVersion, numPos + 1)
 End If
 IsProjectAvailable = (CInt(varVersion)>=FIRST_VERSION_WITH_PROJECTREADING)
End Function

Sub ReloadCurrentFile() 'built-in function does not work
 Dim numLine
 With newEditor()
     .assignActiveEditor
     numLine = .caretY
     .openFile CurrentFile()
     .caretY(numLine)
 End With
End Sub

Sub ResolveConflict()
 With CreateObject("WScript.Shell")
     .Run WithVariablesSubstituted(cCONFLICT_EDITOR, CurrentFile), 1, True
 End With
End Sub

Sub RunCvsCommand(Byval strCommand, Byval strDirectory)
 Dim objLogger
 Set objLogger = New ProcessLogger
 With CreateObject("WScript.Shell")
      .CurrentDirectory = strDirectory
      objLogger.FollowProcess .Exec(WithVariablesSubstituted("""" & sCVS_EXECUTABLE & """ " & strCommand, CurrentFile)), CurrentFile
 End With
End Sub

Sub RunCvsForAll(ByVal strCommand)
 Dim intCount
 Dim objLogger
 Set objLogger = New ProcessLogger
 strCommand = WithInput(strCommand)
 With CreateObject("WScript.Shell")
     For intCount = 0 To projectFilesCount - 1
        .CurrentDirectory = DirectoryOf(projectFiles(intCount))
        objLogger.FollowProcess .Exec(WithVariablesSubstituted("""" & sCVS_EXECUTABLE & """ " & strCommand, projectFiles(intCount))), projectFiles(intCount)
     Next
 End With
End Sub

Function Substitute(strHaystack, strNeedle, strNewHay)
 Dim numPos
 numPos = InStr(1, strHaystack, strNeedle)
 If numPos>0 Then
   strHaystack = Left(strHaystack, numPos - 1) & strNewHay & Mid(strHaystack, numPos + Len(strNeedle))
 End If
 Substitute = strHaystack
End Function

Function WithInput(Byval strCommand)
 Dim numPos
 numPos = InStr(1, strCommand, "%Input%")
 While numPos>0
      strCommand = Substitute(strCommand, "%Input%", InputBox("Comment:"))
      numPos = InStr(1, strCommand, "%Input%")
 Wend
 WithInput = strCommand
End Function

Function WithVariablesSubstituted(Byval strCommand, Byval strFile)
 Dim numPos
 strCommand = Substitute(strCommand, "%FullFileName%", strFile)
 strCommand = Substitute(strCommand, "%File%", strFile)
 numPos = InStrRev(strFile, "\")
 If numPos>0 Then
   strCommand = Substitute(strCommand, "%FilePath%", Left(strFile, numPos))
   strFile = Mid(strFile, numPos + 1)
   strCommand = Substitute(strCommand, "%FileName%", strFile)
   numPos = InStrRev(strFile, ".")
   If numPos>0 Then
     strCommand = Substitute(strCommand, "%FileName2%", Left(strFile, numPos - 1))
   End If
 End If
 WithVariablesSubstituted = WithInput(strCommand)
End Function

Class clsAccesskeyGenerator
 Dim mstrKeys

 Private Sub AddKey(ByVal strKey)
    mstrKeys = mstrKeys & UCase(strKey)
 End Sub

 Private Sub Class_Initialize()
    mstrKeys = ""
 End Sub

 Private Sub HandleGivenAccessKey(ByRef strCommand, ByRef numPos)
    If Not IsKeyKnown(Mid(strCommand, numPos + 1, 1)) Then
      AddKey Mid(strCommand, numPos + 1, 1)
    Else
      strCommand = Left(strCommand, numPos - 1) & Mid(strCommand, numPos + 1)
      numPos = 0 ' remove accesskey, as it was already registered
    End If
 End Sub

 Private Sub DetermineNewAccessKey(ByRef strCommand)
    Dim numPos
    numPos = 1
    While IsKeyKnown(Mid(strCommand, numPos, 1)) And (numPos<=Len(strCommand))
         numPos = numPos + 1
    Wend
    If numPos<=Len(strCommand) Then
      AddKey Mid(strCommand, numPos, 1)
      strCommand = Left(strCommand, numPos - 1) & "&" & Mid(strCommand, numPos)
    End If
 End Sub

 Private Function IsKeyKnown(ByVal strKey)
    IsKeyKnown = InStr(1, mstrKeys, UCase(strKey))>0
 End Function

 Public Function WithAccessKey(ByVal strCommand)
    Dim numPos, strKey
    numPos = InStr(1, strCommand, "&")
    If numPos>0 Then
      HandleGivenAccessKey strCommand, numPos
    End If
    If numPos<1 Then
      DetermineNewAccessKey strCommand
    End If
    WithAccessKey = strCommand
 End Function
End Class

Class ProcessLogger
 Private medtOutput
 Private mnumPrintedCharacters

 Private Sub Class_Initialize()
    Set medtOutput = newEditor()
    medtOutput.assignLog
    medtOutput.text ""
 End Sub

 Public Function FollowProcess(ByVal objProcess, ByVal strFileName)
    Dim strOutput, numResult
    mnumPrintedCharacters = 0
    With objProcess
        While .Status=0
             strOutput = .StdOut.ReadAll
             If Len(strOutput)>mnumPrintedCharacters Then
               medtOutput.appendText Mid(strOutput, mnumPrintedCharacters + 1)
               mnumPrintedCharacters = Len(strOutput)
             End If
        Wend
        numResult = .ExitCode
        FollowProcess = .ExitCode
        If numResult<>0 Then
          medtOutput.appendText " error processing " & strFileName & Chr(10)
        End If
    End With
 End Function
End Class
